"""AirPatrol constants."""

AUTH_BASE_URL = "https://auth.apsrvd.io/v1"
AUTH_LOGIN_ENDPOINT = "/login"
AUTH_PAIRINGS_ENDPOINT = "/pairings"

API_VERSION = "12"
API_BASE_URL = f"https://api.apsrvd.io/{API_VERSION}"
API_COMMAND_ENDPOINT = "/command"
